#
# Copyright 2014 Seth Vargo <sethvargo@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

module Cleanroom
  class Error < StandardError; end

  class InaccessibleError < Error
    def initialize(name, instance)
      @name, @instance = name, instance
    end

    def to_s
<<-EOH.gsub(/\r?\n/, ' ')
Undefined local variable or method `#{@name}' for #{@instance}. It may have
been removed for the purposes of evaluating the DSL or for added security. If
you feel you have reached this message in error, please open an issue.
EOH
    end
  end
end
